import { r as registerInstance, h, g as getElement } from './index-77a9c9e7.js';
import { D as DC, W as WC, u as to, A as Tf, J as Ef, Q as Qm, E } from './index.esm-c2283299.js';
import { a0 as Ln } from './index.esm-f87ff99e.js';
import { s as store } from './redux-store.service-25cf19a5.js';
import './index.esm-78425ae1.js';

const BpSdkQuestionSetNavigationContainer = class {
    constructor(hostRef) {
        registerInstance(this, hostRef);
        this.handleSubscriptions = () => {
            this.storeUnsubscribe && this.storeUnsubscribe();
            store.mapDispatchToProps(this, {
                updateActiveQuestionSet: DC,
                removeActiveAnswerId: WC,
            });
            this.storeUnsubscribe = store.mapStateToProps(this, this.mapStateToProps);
        };
        this.mapStateToProps = (state) => {
            return {
                activeQuestionSetId: to(state),
                areParentQuestionSetsInNavigationEnabled: Tf(state),
                isQuestionSetNavigationEnabled: Ef(state),
                parentQuestionSets: Qm(state),
                questionSetsList: E(state),
            };
        };
        this.activeQuestionSetId = undefined;
        this.areParentQuestionSetsInNavigationEnabled = undefined;
        this.isQuestionSetNavigationEnabled = undefined;
        this.parentQuestionSets = [];
        this.questionSetsList = [];
        this.useExternal = false;
        this.questionSets = [];
    }
    resetState() {
        this.questionSets = [];
    }
    onQuestionSetPropsChange() {
        const insertedParentQuestionSets = new WeakMap();
        if (this.areParentQuestionSetsInNavigationEnabled) {
            this.questionSets = this.questionSetsList
                .filter((questionSet) => Ln(questionSet))
                .reduce((acc, set) => {
                if (set.parentQuestionSetId) {
                    const parentQuestionSet = this.parentQuestionSets.find((parent) => parent.id === set.parentQuestionSetId);
                    if (parentQuestionSet &&
                        !insertedParentQuestionSets.has(parentQuestionSet)) {
                        insertedParentQuestionSets.set(parentQuestionSet, true);
                        // Navigation is based on `question_set_id` so we need to override it on the parent question set
                        // so that it will navigate to the first child question set when clicked
                        acc.push(Object.assign(Object.assign({}, parentQuestionSet), { question_set_id: set.question_set_id, isParent: true }));
                    }
                    acc.push(Object.assign(Object.assign({}, set), { isChild: true }));
                }
                else {
                    acc.push(set);
                }
                return acc;
            }, new Array());
        }
        else {
            this.questionSets = this.questionSetsList;
        }
    }
    connectedCallback() {
        this.resetState();
        this.handleSubscriptions();
    }
    componentWillLoad() {
        this.useExternal = !!customElements.get('bp-sdk-external-question-set-navigation-container');
        customElements
            .whenDefined('bp-sdk-external-question-set-navigation-container')
            .then(() => {
            this.useExternal = true;
        });
        this.handleSubscriptions();
        this.onQuestionSetPropsChange();
    }
    onRemoveActiveAnswerRequest() {
        this.removeActiveAnswerId();
    }
    disconnectedCallback() {
        this.storeUnsubscribe && this.storeUnsubscribe();
    }
    handleActiveQuestionSetUpdate(evt) {
        if (this.useExternal) {
            return;
        }
        evt.detail && this.updateActiveQuestionSet(evt.detail);
        this.removeActiveAnswerId();
    }
    componentWillUpdate() {
        this.handleSubscriptions();
    }
    render() {
        return [
            this.useExternal && this.renderExternal(),
            !this.useExternal && this.renderInternal(),
        ];
    }
    renderExternal() {
        return (h("bp-sdk-external-question-set-navigation-container", { activeQuestionSetId: this.activeQuestionSetId, areParentQuestionSetsInNavigationEnabled: this.areParentQuestionSetsInNavigationEnabled, isQuestionSetNavigationEnabled: this.isQuestionSetNavigationEnabled, parentQuestionSets: this.parentQuestionSets, questionSets: this.questionSets, questionSetsList: this.questionSetsList }));
    }
    renderInternal() {
        return (h("bp-sdk-question-set-navigation", { activeId: this.activeQuestionSetId, questionSets: this.questionSets, isNavigationEnabled: this.isQuestionSetNavigationEnabled }));
    }
    get el() { return getElement(this); }
    static get watchers() { return {
        "questionSetsList": ["onQuestionSetPropsChange"],
        "parentQuestionSets": ["onQuestionSetPropsChange"]
    }; }
};

export { BpSdkQuestionSetNavigationContainer as bp_sdk_question_set_navigation_container };
